
# This is derived from lookat-tree-distances.R
# It combines the DENIM and SB results into one plot each
# for posterior P(true topol) and branch score

source("C:/Users/GRJ/AAA/Programming/Biology/DENIMSimulations/Utils/scenario-dir-names.R")


denim.10spp.fpath <- paste0(analysis.dir, "/Leache10spp-Simp-0p005-30Mplus50rep")
sbeast.10spp.fpath <- paste0(analysis.dir, "/SB-Leache10spp-30Mplus50rep")

denim.4spp.fpath <- paste0(analysis.dir, "/Leache4spp-Simp-0p005-20Mplus")
sbeast.4spp.fpath <- paste0(analysis.dir, "/SB-Leache4spp-20Mplus")


#######################################################################################


plot.boxplot.horizontal <- function(denim.pvals, sb.pvals, title) {
	sdirs <- colnames(denim.pvals)
	par(mfcol = c(1, 1))
	par(mar = c(3, 8, 4, 0) + 0.1)
	
	dsb.pvals <- matrix(0, ncol=3*ncol(denim.pvals), nrow=nrow(denim.pvals))
	scenarios <- labels.for.plot(sdirs)
	for (i in 1:ncol(denim.pvals)) {
		dsb.pvals[, 3*i - 2] <- denim.pvals[,i]
		dsb.pvals[, 3*i - 1] <- sb.pvals[,i]
		dsb.pvals[, 3*i ] <- NA
	}
	
	
	boxplot(dsb.pvals, yaxt = "n", horizontal=TRUE,
					main = title, col=c(gray(.7), gray(.95), gray(1)))
	axis(2, at = 3*(1:length(scenarios))-1, labels = scenarios, las = 1)
}


plot.boxplot.screen.and.pdf.horizontal <- function(denim.pvals, sb.pvals, fpath, title) {
	plot.boxplot.horizontal(denim.pvals, sb.pvals, title)
 	pdf(file=fpath, pointsize=10, width=4.5, height=8)
 	plot.boxplot.horizontal(denim.pvals, sb.pvals, title)
 	dev.off()
}



####################################################################################


sdirs.and.plotorder.spp.4 <- function() {
	list(sdirs=fourspp.all.scenario.dirs, reorder=c(13, 1:12, 14:17))
}

	

#################################################################################################


# Chooses ordering depending on which scenarios are included
sdirs.and.plotorder.spp.10 <- function() {
	nnn <- 0
	if (just.coal.paraphyly) { nnn <- nnn + 1 }
	if (just.leache) { nnn <- nnn + 1 }
	if (just.coal.big.rates) { nnn <- nnn + 1 }
	stopifnot(nnn <= 1)
	
	if (just.coal.paraphyly) {
		sdirs <- tenspp.just.coal.paraphyly.dirs
		reorder <- c(5,1,2,3,4,6,7)
	} else if (just.leache) {
		sdirs <- tenspp.leache.scenario.dirs
		reorder <- 1:19
	} else if (just.coal.big.rates) {
		sdirs <- c("COAL005", "COAL01", "COAL02", "COAL04", "COAL08", "COAL16")
		reorder <- 1:6
	} else {
		sdirs <- tenspp.all.scenario.dirs
		reorder <- c(19, 1:18, 20:25)
	}
	list(sdirs=sdirs, reorder=reorder)
}


sdirs.and.plotorder <- function(nspp) {
	if (nspp == 10) {
		xxx <- sdirs.and.plotorder.spp.10()
		}
	if (nspp == 4) {
		xxx <- sdirs.and.plotorder.spp.4()
	}
	xxx
}





plot.PPts <- function(nspp, denim.fpath, sbeast.fpath) {
	xxx <- sdirs.and.plotorder(nspp)
	
	# read.table() does not read column names with "-" in them properly, they become ".". 
	# So I replace colnames with xxx$sdirs[xxx$reorder]

	denim <-  read.table(paste0(denim.fpath,  "/", "ptrue-topol.txt"), header = TRUE)
	sbeast <- read.table(paste0(sbeast.fpath, "/", "ptrue-topol.txt"), header = TRUE)
	colnames(denim) <- xxx$sdirs[xxx$reorder]
	colnames(sbeast) <- xxx$sdirs[xxx$reorder]
	plot.boxplot.screen.and.pdf.horizontal(denim,	sbeast, paste0(denim.fpath,  "/", "denim-sb-ptrue-topol.pdf"),
																				 "Posterior probability of true topology")
}


plot.branch.scores <- function(nspp, denim.fpath, sbeast.fpath) {
	xxx <- sdirs.and.plotorder(nspp)
	
	# read.table() does not read column names with "-" in them properly, they become ".". 
	# So I replace colnames with xxx$sdirs[xxx$reorder]
	
	denim <-  read.table(paste0(denim.fpath,  "/", "branch-score.txt"), header = TRUE)
	sbeast <- read.table(paste0(sbeast.fpath, "/", "branch-score.txt"), header = TRUE)
	colnames(denim) <- xxx$sdirs[xxx$reorder]
	colnames(sbeast) <- xxx$sdirs[xxx$reorder]
	plot.boxplot.screen.and.pdf.horizontal(denim,	sbeast, paste0(denim.fpath,  "/", "denim-sb-branch-score.pdf"),
																				 "Posterior mean of branch scores")
}




plot.PPts(10, denim.10spp.fpath, sbeast.10spp.fpath)
plot.PPts(4, denim.4spp.fpath, sbeast.4spp.fpath)

plot.branch.scores(10, denim.10spp.fpath, sbeast.10spp.fpath)
plot.branch.scores(4, denim.4spp.fpath, sbeast.4spp.fpath)


















