
library(ape)
library(phangorn)

source("C:/Users/GRJ/AAA/Programming/Biology/DENIMSimulations/Utils/scenario-dir-names.R")

tree.log.analyser.jar <- "java -cp C:/Users/GRJ/AAA/software/BEASTv1.8.3/lib/beast.jar dr.app.tools.TreeLogAnalyser"




treeAnlys.command.one.scenario.one.rep <- function(scenario.dir, rep) {
  tree.fpath <- paste0(BEASTresults.dir, "/",
  										 scenario.rep.fname(scenario.dir, rep), "/species.trees")
  treelist <- read.nexus(tree.fpath, force.multi = TRUE)
  burnin <- burnin.from.Nsamples(length(treelist))
  
  cred.tree.fpath <- paste0(BEASTresults.dir, "/",
  										 scenario.rep.fname(scenario.dir, rep), "/species.cred.trees")
  
  command <- paste0(tree.log.analyser.jar, " -burnin ",  burnin, " -probability 0.95 ", "-export ", cred.tree.fpath, " ", tree.fpath)
  command
# options for TreeLogAnalyser
# - burnin                    "the number of states to be considered as 'burn-in' [default = none]";
# - export file-name          "name of file to export";
# - limit                     "don't export trees with support lower than limit [default = 0.0]";
# - probability               "credible set probability limit [default = 0.95]";
# - max                       "export no more than max trees [default = all]";
# - short                     "use this option to produce a short report";
# - help                      "option to print this message".
}




make.credible.tree.set <- function(cred.tree.fpath) {
	treelist <- read.nexus(cred.tree.fpath,  force.multi = TRUE)
	tree.names <- names(treelist)
	textlines <- readLines(con=cred.tree.fpath)
	textlines <- textlines[grep("tree ", textlines, ignore.case = TRUE)]
	weights <- rep(-1, length(tree.names))
	names(weights) <- tree.names
	for (tn in 1:length(tree.names)) {
		treenam <- paste0(tree.names[tn], " ", collapse="")
		i <- grep(treenam, textlines, fixed=TRUE)
		stopifnot(length(i) ==1)
		x <- strsplit(textlines[i], "[&W", fixed=TRUE)[[1]][2]
		x <- strsplit(x, " ")[[1]]
		for (j in 1:length(x)) {
			if (nchar(x[j]) > 0) {
				weights[tn] <- as.double(x[j])
				break
			}
		}
	}
	cred.tset <- 1:min(which(cumsum(weights)>0.95))
  treelist[cred.tset]	
}


is.in.cred.tree.set <- function(cred.tree.set, true.tree) {
	is.in <- FALSE
	for (i in 1:length(cred.tree.set)) {
		rfd <- RF.dist(true.tree, cred.tree.set[[i]], rooted=TRUE)
		if (rfd == 0) {
			is.in = TRUE
		}
	}
	is.in
}




do.one.cred.set <- function(scenario.dir, rep, nspp) {
	cred.tree.fpath <- paste0(BEASTresults.dir, "/",
														scenario.rep.fname(scenario.dir, rep), "/species.cred.trees")
	cred.tree.set <- make.credible.tree.set(cred.tree.fpath)
	# Loaded into Beauti, the default taxon sets are a,b,c,d. They 'should' be A,B,C,D.
	# Better to use 'after ^' in TaxonSets. Can fix here if needed.
	#true.tree <- read.tree(text="(((a,b),c),d);")
	if (nspp == 4) {
		true.tree <- read.tree(text="(((A,B),C),D);")
	} else {
		true.tree <- read.tree(text="((((((A,B),C),D)  ,  (((E,F),G),H)), I), J);")
	}
	ok <- is.in.cred.tree.set(cred.tree.set, true.tree)
	cat(scenario.dir, rep, ok, "\n")
	cat(scenario.dir, rep, ok, "\n", file=fpath.for.coverage(), append=TRUE)
	if (ok) {
		1
	} else {
		0
	}
}




do.credsets <- function(scenario.directories, nspp) {
	cat("Coverage\n", file=fpath.for.coverage())
	
	for (scenario in scenario.directories) {
		for (rep in first.replicate:last.replicate) {
			command <- treeAnlys.command.one.scenario.one.rep(scenario, rep)
			system(command)
		} 
	}
	
	
	for (scenario in scenario.directories) {
		ntrue <- 0
		for (rep in first.replicate:last.replicate) {
			ntrue <- ntrue + do.one.cred.set(scenario, rep, nspp)
		} 
	cat(ntrue, "\n\n")
	cat(ntrue, "\n\n", file=fpath.for.coverage(), append=TRUE)
	}
}



####################################################################################




do.credsets.spp.4  <- function() {
	do.credsets(fourspp.all.scenario.dirs, 4)
}





########################################################################################


do.credsets.spp.10  <- function() {
	nnn <- 0
	if (just.coal.paraphyly) { nnn <- nnn + 1 }
	if (just.leache) { nnn <- nnn + 1 }
	if (just.coal.big.rates) { nnn <- nnn + 1 }
	stopifnot(nnn <= 1)
	
	if (just.coal.paraphyly) {
		stopifnot(just.leache == FALSE)
		do.credsets(tenspp.just.coal.paraphyly.dirs, 10)
	} else if (just.leache) {
		do.credsets(tenspp.leache.scenario.dirs, 10)
	} else if (just.coal.big.rates) {
		sdirs <- c("COAL005", "COAL01", "COAL02", "COAL04", "COAL08", "COAL16")
		do.credsets(sdirs, 10)
	} else {
		do.credsets(tenspp.all.scenario.dirs, 10)
	}
}




###########################################################################################



do.coverage.credsets <- function(nspp) {
	if (nspp == 10) {
		do.credsets.spp.10()
	}
	if (nspp == 4) {
		do.credsets.spp.4()
	}
}


