library(coda)
library(MASS)
library(lattice)



source("C:/Users/GRJ/AAA/Programming/Biology/DENIMSimulations/Utils/scenario-dir-names.R")





read.log <- function(fpath) {
  x <- read.table(fpath, header=TRUE)
  burnin <- burnin.from.Nsamples(nrow(x))
  xb <- x[burnin:nrow(x),]
  mcmclog <- as.mcmc(xb)
  mcmclog
  }



# For one scenario, this makes two matrices, each with one row per locus
# and one per replicate (so usually 10 by 100). est.migcounts[,] contains the
# mean (over MCMC samples) of the count of number of migration events. 
# est.Pmig[,] contains the posterior probability of one or more migration events.
est.migs <- function(sdir) {
	est.migcounts <- matrix(0, nrow=number.of.loci, ncol=number.of.replicates) 
	est.Pmig <- matrix(0, nrow=number.of.loci, ncol=number.of.replicates) 
	for (rep in first.replicate:last.replicate) {
		params.fpath <- paste0(beast.dir.from.scenario.dir.and.rep(sdir, rep), "/", "locus1.log")
		mcmc <- read.log(params.fpath)
		migcounts <- mcmc[, paste0("MigCount.gTreeWE.t.locus", 1:number.of.loci)]
    migcount.means <- colMeans(migcounts[,])
    nmcmc.samples <- nrow(migcounts)
    for (loc in 1:number.of.loci) {
    	nyes <- length(which(migcounts[,loc] > 0)) 
    	est.Pmig[loc, rep] <- nyes / nmcmc.samples
    }
    est.migcounts[ , rep] <- migcount.means
	}
	list(est.migcounts=est.migcounts, est.Pmig=est.Pmig)
}







# This reads the "migs.txt" file for a scenario. (This file was made by my version
# of MCcoal.) It uses this to make a matrix true.migcounts[,] which is comparble to
# est.migcounts[,] made by est.migs.
true.migs <- function(sdir) {
	true.migcounts <- matrix(0, nrow=number.of.loci, ncol=number.of.replicates) 
	
	if (is.timezero.allele(sdir)) {
		true.migcounts[1,] <- 1
	} else if (is.timezero.mig(sdir)) {
		true.migcounts[,] <- 1
	} else {
		for (rep in first.replicate:last.replicate) {
			true.migs.fpath <- paste0(beast.dir.from.scenario.dir.and.rep(sdir, rep), "/", "migs.txt")
			true.migs.lines <- readLines(con=true.migs.fpath)
			true.mig.tokens <- strsplit(true.migs.lines, split="[[:blank:]]+")
			for (loc in 1:length(true.migs.lines)) {
				n <- length(true.mig.tokens[[loc]])
				stopifnot(n >= 2)
				n <- n - 2
				stopifnot(n %% 5 == 0)
				n <- n/5
				true.migcounts[loc, rep] <- n
			}
		}
	}
	true.migcounts
}



# This compares the true and estimated counts of migration events.
# It returns
# no: the estimated number of of migration events when the true number is zero
# yes: the estimated number of of migration events when the true number is >= 1
# estyes0: posterior probability of one or more migration events when the true number is zero
# estyesSome: posterior probability of one or more migration events when the true number is zero
# truyes: the true number of (locus,rep) pairs which contain a migration events
# truno: the true number of (locus,rep) pairs which do not contain a migration event
yesno.means <- function(sdir) {
	estmigs <- est.migs(sdir)
	est.mig.matrix <- estmigs$est.migcounts
	true.mig.matrix <- true.migs(sdir)
	ests0 <- double(0)
	estsSome <- double(0)
	estyes0 <- 0
	estyesSome <- 0
	truno <- 0
	truyes <- 0
	for (rep in first.replicate:last.replicate) {
		for (loc in 1:number.of.loci) {
			tc <- true.mig.matrix[loc,rep] 
			if (tc == 0) {
				ests0[length(ests0) + 1] <- est.mig.matrix[loc,rep]
				estyes0 <- estyes0 + (estmigs$est.Pmig[loc,rep] >= 0.5)
				truno <- truno + 1
			} else {
				estsSome[length(estsSome) + 1] <- est.mig.matrix[loc,rep]
				estyesSome <- estyesSome + (estmigs$est.Pmig[loc,rep] >= 0.5)
				truyes <- truyes + 1
			}
		}
	}
	list(no=ests0, yes=estsSome, estyes0=estyes0, estyesSome=estyesSome, truyes=truyes, truno=truno)
}


# Makes a simple table of the false positives and correct detections of one or more migration events
make.table  <- function(sdirs) {
	xxx <- NULL
	for (sdir in sdirs) {
		xx <- yesno.means(sdir)
		xxx <- c(xxx, list(xx))
		txt <- paste0(sdir, "    ",  xx$estyes0, "/", xx$truno, "    ", xx$estyesSome, "/", xx$truyes, "\n")
		cat(txt)
	}
	sink(fpath.for.mig.detection())
	for (s in 1:length(sdirs)) {
		xx <- xxx[[s]]
		txt <- paste0(sdirs[s], "    ",  xx$estyes0, "/", xx$truno, "    ", xx$estyesSome, "/", xx$truyes, "\n")
		cat(txt)
	}
	sink(NULL)
}



# draws a set of boxplots of estimated migration event counts.
draw.boxplots.array <- function(layout.mat, sdirs) {
	layout(mat=layout.mat)
	par(mar=c(4,4,3,1)+.1)
	for (sdir in sdirs) {
		cat(sdir, "... ")
		xx <- yesno.means(sdir)
		
		maxyes <- 0
		if (length(xx$yes) > 0) {
			maxyes <- max(xx$yes) * 1.02
		}
		if (maxyes < 2.0) {
			maxyes <- 2.0
		}
		
		
		boxplot(list(xx$no, xx$yes), 
						width=1+c(length(xx$no), length(xx$yes)), 
						xaxt="n", ylim=c(0,maxyes))
		axis(1, at=1:2, labels=c("0", ">=1"))
		title(main = label.for.plot(sdir))
	}
}




draw.boxplots.array.screen.pdf.and.cat.table <- function(layout.mat, sdirs) {
	# draw.boxplots.array(layout.mat, sdirs)
	# pdf(file=fpath.for.mig.yes.no(), pointsize=9, width=6, height=8)
	# draw.boxplots.array(layout.mat, sdirs)
	# dev.off()
	make.table(sdirs)
}


####################################################################################

do.migs.yes.no.spp.4 <- function() {
	layout.mat <- matrix(c(13,0,0,0, 1,2,3,4, 5,6,7,8, 9,10,11,12, 14,15,16,17), nrow=5, ncol=4, byrow=TRUE)
	draw.boxplots.array.screen.pdf.and.cat.table(layout.mat, fourspp.all.scenario.dirs)
}






# Chooses layout depending on which scenarios are included, calls
# draw.boxplots.array.screen.pdf.and.cat.table(). 
# The main set of scenarios is tenspp.all.scenario.dirs
do.migs.yes.no.spp.10 <- function() {
	
	nnn <- 0
	if (just.coal.paraphyly) { nnn <- nnn + 1 }
	if (just.leache) { nnn <- nnn + 1 }
	if (just.coal.big.rates) { nnn <- nnn + 1 }
	stopifnot(nnn <= 1)
	
	
	if (just.coal.paraphyly) {
		layout.mat <- matrix(c(5,0,  1,2, 3,4, 6,7), nrow=4, ncol=2, byrow=TRUE)
		draw.boxplots.array.screen.pdf.and.cat.table(layout.mat, tenspp.just.coal.paraphyly.dirs)
	} else if (just.leache) {
		stop("Not implemented in do.migs.yes.no.spp.10()")
	} else if (just.coal.big.rates) {
		stop("Not implemented in do.migs.yes.no.spp.10()")
	}	else {
		layout.mat <- matrix(c(19,0,0,0,0,0,  1,2,3,4,5,6,  7,8,9,10,11,12,  13,14,15,16,17,18,  20,21,22,23,24,25), nrow=5, ncol=6, byrow=TRUE)
		draw.boxplots.array.screen.pdf.and.cat.table(layout.mat, tenspp.all.scenario.dirs)
	}
}


#################################################################################################


do.migs.yes.no <- function(nspp) {
	if (nspp == 4) {
		do.migs.yes.no.spp.4()
	}
	if (nspp == 10) {
		do.migs.yes.no.spp.10()
	}
}




