
# This uses MCcoal files supplied by Adam Leache.
# It copies them into my directory structure,
# then runs MCcoal (My-MCcoal) on them,
# then renames the sequence and tree files to my conventions,
# then makes the time-zero ones



##################################################################################
# File paths and names

this.directory <- "C:/Users/GRJ/AAA/Programming/Biology/DENIMSimulations/SB-Leache-FourSpp"
MyMCcoal.fpath <- "C:/Users/GRJ/AAA/Programming/Biology/DENIMSimulations/Utils/My-MCcoal.exe"


source("C:/Users/GRJ/AAA/Programming/Biology/DENIMSimulations/Utils/scenario-dir-names.R")



leache.rate.dirs <- c("M0.001", "M0.01", "M0.1", "M1",
									"M0.001", "M0.01", "M0.1", "M1", 
									"M0.001", "M0.01", "M0.1", "M1")

leache.pat.names <- c("Anc", "Anc", "Anc", "Anc",
											"AB", "AB", "AB", "AB",
									    "BC", "BC", "BC", "BC")



leache.names <- paste0(leache.rate.dirs, "/", leache.pat.names)


my.dpath <- function(migpat, rep) {
	my.repnum <- sprintf("%03d", rep)
	paste0(BEASTresults.dir, "/", fourspp.all.scenario.dirs[migpat], "/rep", my.repnum)
}


my.nomig.dpath <- function(rep) {
	my.dpath(13, rep)
}


leache.ctrl.fpath <- function(migpat, rep) {
	leache.repnum <- sprintf("%d", rep-1)
	paste0(this.directory, "/ZCtrlFiles/", leache.names[migpat], leache.repnum, ".ctl")
}



leache.nomig.ctrl.fpath <- function(rep) {
	leache.repnum <- sprintf("%d", rep-1)
	paste0(this.directory, "/ZCtrlFiles/", paste0("M0.001/coal"), leache.repnum, ".ctl")
}


leache.trees.leaf <- function(migpat, rep) {
	leache.repnum <- sprintf("%d", rep-1)
	paste0(leache.pat.names[migpat], leache.repnum, ".tre")
}

leache.seq.leaf <- function(migpat, rep) {
	leache.repnum <- sprintf("%d", rep-1)
	paste0(leache.pat.names[migpat], leache.repnum, ".txt")
}

leache.nomig.trees.leaf <- function(rep) {
	leache.repnum <- sprintf("%d", rep-1)
	paste0("coal", leache.repnum, ".tre")
}

leache.nomig.seq.leaf <- function(rep) {
	leache.repnum <- sprintf("%d", rep-1)
	paste0("coal", leache.repnum, ".txt")
}


#################################################################################
# copying from Leache's ctrl file names to mine


# for s <= 12:
# Copy control files from things like
# /ZCtrlFiles/M0.001/AB7.ctl     to      IM-0p001/rep008/ctrl.txt

# for s=13 to 17:
# Copy control files from things like
# /ZCtrlFiles/M0.001/coal7.ctl      to       <my dir>/rep008/ctrl.txt
copy.ctrl.files <- function(s) {
	all.ok <- TRUE
	for (rep in first.replicate:last.replicate) {
		if (s < fourspp.nof.mccoal.scenarios) {
			from <- leache.ctrl.fpath(s, rep)
		} else {
			from <- leache.nomig.ctrl.fpath(rep)
		}
		to <- my.dpath(s, rep)
		if (!dir.exists(to)) {
			dir.create(to)
		}
		to <- paste0(to, "/", "ctrl.txt")
		ok <- file.copy(from, to, overwrite=TRUE)
		if (!ok) {
			cat("Failed to copy ", from, "\n", to, "\n")
			all.ok <- FALSE
			break
		}
	}
	stopifnot(all.ok)
	all.ok
}



#################################################################################
# Running MCCoal, and renamng output to my conventions


run.mccoal.one.scenario.one.rep <- function(scenario.dir, rep) {
	repname <- sprintf("rep%03d", rep)
  working.dir <- paste0(BEASTresults.dir, "/", scenario.dir, "/", repname)
  setwd(working.dir)
  command <- paste0(MyMCcoal.fpath, " ctrl.txt")
  error.code <- system(command)
  if (error.code != 0) {
  	err.message <- paste("Error. Code=",  error.code,
  											 "running MCcoal with directory", scenario.dir, "rep", rep)
    print(err.message)
  }
}




# Runs MCcoal on main migration pattern s (in 1:12) and renames the output
# sequence and tree files  (AB3.txt to seq.txt, AB3.tre to trees.txt)
run.mccoal.main.migs <- function(s) {
	all.ok <- TRUE
	scenario <- fourspp.all.scenario.dirs[s]
	for (rep in first.replicate:last.replicate) {
		run.mccoal.one.scenario.one.rep(scenario, rep)
		dpath <- my.dpath(s, rep)
		
		from <- paste0(dpath, "/", leache.seq.leaf(s, rep))
		to <- paste0(my.dpath(s, rep), "/", "seq.txt")
		ok <- file.rename(from, to)
		if (!ok) {
			cat("Failed to rename\n ", from, "\n", to, "\n")
			all.ok <- FALSE
			break
		}
		
		from <- paste0(dpath, "/", leache.trees.leaf(s, rep))
		to <- paste0(my.dpath(s, rep), "/", "trees.txt")
		ok <- file.rename(from, to)
		if (!ok) {
			cat("Failed to rename\n ", from, "\n", to, "\n")
			all.ok <- FALSE
			break
		}	
	}
	stopifnot(all.ok)
	all.ok
}


# Runs MCcoal on no-migration pattern and renames the output
# sequence and tree files  (coal3.txt to seq.txt, coal3.tre to trees.txt)
run.mccoal.nomig <- function() {
	all.ok <- TRUE
	for (rep in first.replicate:last.replicate) {
		run.mccoal.one.scenario.one.rep(fourspp.no.mig.scenario.dir, rep)
		dpath <- my.nomig.dpath(rep)
		from <- paste0(dpath, "/", leache.nomig.seq.leaf(rep))
		to <- paste0(my.nomig.dpath(rep), "/", "seq.txt")
		ok <- file.rename(from, to)
		if (!ok) {
			cat("Failed to rename\n ", from, "\n", to, "\n")
			all.ok <- FALSE
			break
		}
		
		from <- paste0(dpath, "/", leache.nomig.trees.leaf(rep))
		to <- paste0(my.nomig.dpath(rep), "/", "trees.txt")
		ok <- file.rename(from, to)
		if (!ok) {
			cat("Failed to rename\n ", from, "\n", to, "\n")
			all.ok <- FALSE
			break
		}	
	}
	stopifnot(all.ok)
	all.ok
}



######################################################################################################
# Making the 
# "Nonsister-1-allele", "Nonsister-1-mig", "Sister-1-allele", "Sister-1-mig"
# sequence files from the No-mig ones

B4.to.A5 <- function(xx, one) {
	b1i <- grep("b1^B", xx, fixed = TRUE)
	b2i <- grep("b2^B", xx, fixed = TRUE)
	b3i <- grep("b3^B", xx, fixed = TRUE) 
	b4i <- grep("b4^B", xx, fixed = TRUE)
	if (one) {
		b1i <- b1i[1]
		b2i <- b2i[1]
		b3i <- b3i[1]
		b4i <- b4i[1]
	}
	b1 <- xx[ b1i ]
	b2 <- xx[ b2i ]
	b3 <- xx[ b3i ]
	b4 <- xx[ b4i ]
	a5 <- gsub("b4^B", "a5^A", b4, fixed = TRUE)
	xx[b1i] <- a5
	xx[b2i] <- b1
	xx[b3i] <- b2
	xx[b4i] <- b3
	xx
}





B4.to.C5 <- function(xx, one) {
	c1i <- grep("c1^C", xx, fixed = TRUE)
	c2i <- grep("c2^C", xx, fixed = TRUE)
	c3i <- grep("c3^C", xx, fixed = TRUE) 
	c4i <- grep("c4^C", xx, fixed = TRUE)
	b4i <- grep("b4^B", xx, fixed = TRUE)
	if (one) {
		c1i <- c1i[1]
		c2i <- c2i[1]
		c3i <- c3i[1]
		c4i <- c4i[1]
		b4i <- b4i[1]
	}
	c1 <- xx[ c1i ]
	c2 <- xx[ c2i ]
	c3 <- xx[ c3i ]
	c4 <- xx[ c4i ]
	b4 <- xx[ b4i ]
	c5 <- gsub("b4^B", "c5^C", b4, fixed = TRUE)
	xx[b4i] <- c1
	xx[c1i] <- c2
	xx[c2i] <- c3
	xx[c3i] <- c4
	xx[c4i] <- c5
	xx
}


one.B4.to.A5 <- function(xx) {
	B4.to.A5(xx, TRUE)
}


one.B4.to.C5 <- function(xx, one) {
	B4.to.C5(xx, TRUE)
}



all.B4.to.A5 <- function(xx) {
	B4.to.A5(xx, FALSE)
}



all.B4.to.C5 <- function(xx) {
	B4.to.C5(xx, FALSE)
}


sink.lines <- function(yy, fpath) {
	sink(fpath)
	cat("") # overwrites old file if any
	for (i in 1:length(xx)) {
		cat(yy[i], "\n", append = TRUE, sep="")
	}
	sink(NULL)
}



make.rep.dir <- function(s, rep) {
	dpath <- my.dpath(s, rep)
	if (!dir.exists(dpath)) {
		dir.create(dpath)
	}
}


######################################################################################


for (s in 1:length(fourspp.all.scenario.dirs)) {
	dpath <- paste0(BEASTresults.dir, "/", fourspp.all.scenario.dirs[s])
	if (!dir.exists(dpath)) {
		dir.create(dpath)
	}
}





# Copy control files. Go from Leache's to my conventions
for (s in 1:length(fourspp.all.scenario.dirs)) {
	ok <- copy.ctrl.files(s)
	if (!ok) {
		break
	}
}



# Run MCcoal on the ctrl files just copied, and rename the output
# sequence and tree files to my conventions
for (s in 1:fourspp.nof.migrating.mccoal.scenarios) {
	ok <- run.mccoal.main.migs(s)
	if (!ok) {
		break
	}
}
run.mccoal.nomig()






# Make the time-zero sequences
for (rep in first.replicate:last.replicate) {
	xx <- readLines(paste0(my.dpath(13,rep), "/seq.txt"))
	# 14 "Nonsister-1-allele", 
	# 15 "Nonsister-1-mig", 
	# 16 "Sister-1-allele",
	# 17 "Sister-1-mig"
	make.rep.dir(14,rep)
	sink.lines(one.B4.to.C5(xx), paste0(my.dpath(14,rep), "/seq.txt"))
	make.rep.dir(15,rep)
	sink.lines(all.B4.to.C5(xx), paste0(my.dpath(15,rep), "/seq.txt"))
	make.rep.dir(16,rep)
	sink.lines(one.B4.to.A5(xx), paste0(my.dpath(16,rep), "/seq.txt"))
	make.rep.dir(17,rep)
	sink.lines(all.B4.to.A5(xx), paste0(my.dpath(17,rep), "/seq.txt"))
	cat("Done time-zero sequences for rep", rep, "\n")
}



