
# Reads Phylip format from MCCOAL, like this
# 
# 8 1000 
# a1^A        GCGGGATACG CCGACGTTGC TAAGAGACAT CAAG
# a2^A        GCGGGATACG CCGACGTTGC TAAGAGACAT CCAG
#  ...
# d2^D        GCGGGATACG CCGACGTTGC TAAGAGACAT CCAG
# ...
# 8 1000 
# a1^A        AAACGATCGA
# ... 


# and writes nex files, one per locus of BEAST XML like this

# #NEXUS
# [comment]
# BEGIN DATA;
# DIMENSIONS  NTAX = 8  NCHAR=1000;
# FORMAT DATATYPE = DNA GAP = - MISSING = ?;
# MATRIX	
# a1^A  		ATGCTGGAGGTATTACAAT
# a2^A 	  	ATGCTGGTGGTATTACAAT
# ....
# d2^D  		ATGCTGGTGGTATTACAAT;
# END;



convert.mcoal.seq.nex <- function(dpath, mcseqs.fpath) {
  mcseqs.lines <- readLines(con = mcseqs.fpath)
  mcseqs.lines <- trimws(mcseqs.lines)
  mcseqs.lines <- mcseqs.lines[which(nchar(mcseqs.lines) >0)]
  
  ln <- 1
  locus <- 1
  while (ln < length(mcseqs.lines)) {
    nex.fname <- paste0(dpath, "/locus", locus, ".nex")
    cat("#NEXUS\n[locus ", locus, "]\nBEGIN DATA;\n", file=nex.fname)
    counts <- strsplit(mcseqs.lines[ln], "[[:blank:]]")[[1]]
    counts <- as.integer(counts)
    stopifnot(length(counts) == 2)
    stopifnot(is.finite(counts[1]))
    stopifnot(is.finite(counts[2]))
    stopifnot(counts[1] > 0)
    stopifnot(counts[2] > 0)
    cat("DIMENSIONS  NTAX =", counts[1], "NCHAR =", counts[2], ";\n", file=nex.fname, append=TRUE)
    cat("FORMAT DATATYPE = DNA GAP = - MISSING = ?;\nMATRIX\n", file=nex.fname, append=TRUE)
    for (s in 1:counts[1]) {
      seq.line <- mcseqs.lines[ln + s]
      seq.bits <- strsplit(seq.line, "[[:blank:]]")[[1]]
      taxon <- seq.bits[1]
      seq <- paste0(seq.bits[-1], collapse="")
      stopifnot(nchar(seq) == counts[2])
      cat("\t", taxon, "\t", seq, "\n", file=nex.fname, append=TRUE)
    }
    ln <- ln + counts[1] + 1
    locus <- locus + 1 
    cat(";\nEND;\n", file=nex.fname, append=TRUE)
  }
}

dpath <- "C:/Users/GRJ/AAA/Programming/Biology/DENIMSimulations/Leache-TenSpp/MCcoalToNEX/10spp-seq"


convert.mcoal.seq.nex(dpath, paste0(dpath, "/seq.txt"))



