
# To be source'd in by other R scripts in Leache-FourSpp, Leache-TenSpp, Analysis. 
# It supplies lists of directories and a few helper functions

library(stringr)


analysis.dir <- "C:/Users/GRJ/AAA/Programming/Biology/DENIMSimulations/Analysis"


# These ones used for article
analysis.patterns <- matrix(c(
	"G:/Leache4spp-Simp-0p005-20Mplus",                4,  50, 
	"G:/Leache10spp-Simp-0p005-30Mplus50rep", 10,  50,
	
	"G:/SB-Leache4spp-20Mplus",                  4,  50,
	"G:/SB-Leache10spp-30Mplus50rep",         10,  50,
	
	"G:/Leache4spp-Flex-0p00125", 4,   25,
	"G:/Leache4spp-Flex-0p005",   4,   25,
	"G:/Leache4spp-Flex-0p02",    4,   25,
	"G:/Leache4spp-Flex-0p08",    4,   25
	
),
nrow=8, ncol=3, byrow=TRUE)
colnames(analysis.patterns) <- c("dpath", "nspp", "nreps")


#######################################
# This needs to be set manually for each set of analyses (DENIM vs SB, 4 vs 10, as analysis.patterns[,])

analysis.pattern.index <- 1

######################################




BEASTresults.dir <- analysis.patterns[analysis.pattern.index,"dpath"]

burnin.from.Nsamples <- function(Nsamples) {
	as.integer(round(0.2*Nsamples))
}
	

first.replicate <- 1
last.replicate <- as.integer(analysis.patterns[analysis.pattern.index, "nreps"])

number.of.species <- as.integer(analysis.patterns[analysis.pattern.index, "nspp"])

# Rest generally constant.

just.coal.paraphyly <- FALSE
just.leache <- FALSE
just.coal.big.rates <- FALSE



number.of.loci <- 10
number.of.replicates <- last.replicate + 1 - first.replicate


###########################################################################

fourspp.all.scenario.dirs <- c("AB-C_0p001", "AB-C_0p01", "AB-C_0p1", "AB-C_1p0", 
														"A-B_0p001", "A-B_0p01", "A-B_0p1", "A-B_1p0", 
														"B-C_0p001", "B-C_0p01", "B-C_0p1", "B-C_1p0",
														"NoMig",
														"B-C-1-allele",
														 "B-C-1-mig",
														 "B-A-1-allele",
														 "B-A-1-mig")

fourspp.nof.migrating.mccoal.scenarios <- 12
fourspp.nof.mccoal.scenarios <- 13
fourspp.no.mig.scenario.dir <- fourspp.all.scenario.dirs[13]
# NoMig is the last


###############################################################################

ten.spp.mapping <- matrix(c(
	"ABCD-EFGH_0p01/run",                           "run",             "ABCD-EFGH_0p01", 
  "ABCD-EFGH_0p1/run",                            "run",             "ABCD-EFGH_0p1", 
	"ABCD-EFGH_1p0/run",                            "run",             "ABCD-EFGH_1p0", 
	"E-F_0p01/run",                                 "run",             "E-F_0p01", 
	"E-F_0p1/run",                                  "run",             "E-F_0p1", 
	"E-F_1p0/run",                                  "run",             "E-F_1p0", 
	"E-F_E-G_E-H_F-G_F-H_G-H_0p01/run",             "migEFGH_0.1_run", "E-F_E-G_E-H_F-G_F-H_G-H_0p01",
	"E-F_E-G_E-H_F-G_F-H_G-H_0p1/migEFGH_0.1_run",  "migEFGH_0.1_run", "E-F_E-G_E-H_F-G_F-H_G-H_0p1",
	"E-F_E-G_E-H_F-G_F-H_G-H_1p0/migEFGH_1_run",    "migEFGH_1_run",   "E-F_E-G_E-H_F-G_F-H_G-H_1p0",
	"EF-G_0p01/run",                                "run",             "EF-G_0p01",
	"EF-G_0p1/run",                                 "run",             "EF-G_0p1",
	"EF-G_1p0/run",                                 "run",             "EF-G_1p0",
	"F-G_0p01/run",                                 "run",             "F-G_0p01",
	"F-G_0p1/run",                                  "run",             "F-G_0p1",
	"F-G_1p0/run",                                  "run",             "F-G_1p0",
	"H-I_0p01/run",                                 "migHI_0.1_run",   "H-I_0p01",
	"H-I_0p1/migHI_0.1_run",                        "migHI_0.1_run",   "H-I_0p1",
	"H-I_1p0/migHI_1_run",                          "migHI_1_run",     "H-I_1p0",
	"COAL/nomig_run",                               "nomig_run",       "COAL"),
	nrow=19, ncol=3, byrow=TRUE)
colnames(ten.spp.mapping) <- c("leache.to.repnum", "leaf.in.ctrl", "my.dir")


tenspp.time.zero.dirs <- c(
	"D-E-1-allele", "D-E-1-mig",
	"F-E-1-allele", "F-E-1-mig",
	"F-G-1-allele", "F-G-1-mig")



nof.tenspp.migpats <- nrow(ten.spp.mapping) + length(tenspp.time.zero.dirs)
tenspp.all.scenario.dirs <- c(ten.spp.mapping[,3], tenspp.time.zero.dirs)
tenspp.just.coal.paraphyly.dirs <- tenspp.all.scenario.dirs[c(7,8, 16,17, 19, 20,21)]
tenspp.leache.scenario.dirs <- tenspp.all.scenario.dirs[c(2,3, 5,6, 8,9, 11,12, 14,15, 17,18, 19, 20,21,22,23,24,25)]


scenario.labels.for.plot <- c(
	"AB-C_0.001",	"AB-C_0.01",	"AB-C_0.1",	"AB-C_1.0",
	"A-B_0.001",	"A-B_0.01",	"A-B_0.1",	"A-B_1.0",
	"B-C_0.001",	"B-C_0.01",
	"B-C_0.1",	"B-C_1.0",
	"NoMig",
	"B-C-1-allele",	"B-C-1-mig",	"B-A-1-allele",	"B-A-1-mig",
	
	"DeepAnc_0.01",	"DeepAnc_0.1",	"DeepAnc_1.0",
	"E-F_0.01",	"E-F_0.1",	"E-F_1.0",
	"4-island_0.01",	"4-island_0.1",	"4-island_1.0",
	"EF-G_0.01",	"EF-G_0.1",	"EF-G_1.0",
	"F-G_0.01",	"F-G_0.1",	"F-G_1.0",
	"H-I_0.01",	"H-I_0.1",	"H-I_1.0",
	"NoMig",
	"D-E-1-allele",	"D-E-1-mig",	"F-E-1-allele",	"F-E-1-mig",	"F-G-1-allele",	"F-G-1-mig"
)

names(scenario.labels.for.plot) <- c(
	"AB-C_0p001",	"AB-C_0p01",	"AB-C_0p1",	"AB-C_1p0",
	"A-B_0p001",	"A-B_0p01",	"A-B_0p1",	"A-B_1p0",
	"B-C_0p001",	"B-C_0p01",
	"B-C_0p1",	"B-C_1p0",
	"NoMig",
	"B-C-1-allele",	"B-C-1-mig",	"B-A-1-allele",	"B-A-1-mig",
	"ABCD-EFGH_0p01",	"ABCD-EFGH_0p1",	"ABCD-EFGH_1p0",
	"E-F_0p01",	"E-F_0p1",	"E-F_1p0",
	"E-F_E-G_E-H_F-G_F-H_G-H_0p01",	"E-F_E-G_E-H_F-G_F-H_G-H_0p1",	"E-F_E-G_E-H_F-G_F-H_G-H_1p0",
	"EF-G_0p01",	"EF-G_0p1",	"EF-G_1p0",
	"F-G_0p01",	"F-G_0p1",	"F-G_1p0",
	"H-I_0p01",	"H-I_0p1",	"H-I_1p0",
	"COAL",
	"D-E-1-allele",	"D-E-1-mig",	"F-E-1-allele",	"F-E-1-mig",	"F-G-1-allele",	"F-G-1-mig"
)


##################################################################################


scenario.rep.fname <- function(scenario.dir, rep) {
	stopifnot(is.character(scenario.dir))
	stopifnot(rep >= first.replicate)
	stopifnot(rep <= last.replicate)
	paste0(scenario.dir, "/", "rep", sprintf("%03d", rep))
}




beast.dir.from.scenario.dir.and.rep <- function(scenario.dir, rep) {
	paste0(BEASTresults.dir, "/", scenario.rep.fname(scenario.dir, rep))
}



is.timezero.allele <- function(sdir) {
	substr(sdir, nchar(sdir)-6, nchar(sdir)) == "-allele"
}


is.timezero.mig <- function(sdir) {
	substr(sdir, nchar(sdir)-3, nchar(sdir)) == "-mig"
}




label.for.plot<- function(sdir) {
	
	scenario.labels.for.plot[sdir]
	
	# label <- sdir
	# 
	# if (sdir == "E-F_E-G_E-H_F-G_F-H_G-H_0p01") {
	# 	label <- "4-island_0p01"
	# } else if (sdir == "E-F_E-G_E-H_F-G_F-H_G-H_0p1") {
	# 	label <- "4-island_0p1"
	# }
	# else if (sdir == "E-F_E-G_E-H_F-G_F-H_G-H_1p0") {
	# 	label <- "4-island_1p0"
	# }
	# 
	# 
	# if (sdir == "ABCD-EFGH_0p01") {
	# 	label <- "DeepAnc_0p01"
	# } else if (sdir == "ABCD-EFGH_0p1") {
	# 	label <- "DeepAnc_0p1"
	# }
	# else if (sdir == "ABCD-EFGH_1p0") {
	# 	label <- "DeepAnc_1p0"
	# }
	# 
	# "ABCD-EFGH_0p01"
	# 
	# 
	# 
	# label <- sub("1p0", "1.0", label)
	# label <- sub("0p1", "0.1", label)
	# label <- sub("0p0", "0.0", label)
	# label
}


labels.for.plot<- function(sdirs) {
	labs <- rep("", length(sdirs))
	for (i in 1:length(sdirs)) {
		labs[i] <- label.for.plot(sdirs[i])
	}
	labs
}




analysis.subdirname <- function() {
	xx <- strsplit(BEASTresults.dir, "/")[[1]]
	xx[length(xx)]    ## something like "DENIM-Leache4spp-July17"
}


analysis.subdirpath <- function() {
	paste0(analysis.dir, "/",  analysis.subdirname())
}

make.subdir.if.needed  <- function() {
	dpath <- analysis.subdirpath()
	if (!dir.exists(dpath)) {
		dir.create(dpath)
	}
}


fpath.for.ESS <- function() {
	paste0(analysis.subdirpath(), "/", "ESSs.txt")
}


fpath.for.coverage <- function() {
	paste0(analysis.subdirpath(), "/", "coverage.txt")
}


fpath.for.branch.scores.pdf <- function() {
	paste0(analysis.subdirpath(), "/", "branch-score.pdf")
}

fpath.for.ptrue.topol.pdf <- function() {
	paste0(analysis.subdirpath(), "/", "ptrue-topol.pdf")
}

fpath.for.branch.scores.txt <- function() {
	paste0(analysis.subdirpath(), "/", "branch-score.txt")
}

fpath.for.ptrue.topol.txt <- function() {
	paste0(analysis.subdirpath(), "/", "ptrue-topol.txt")
}



fpath.for.mig.yes.no <- function() {
	paste0(analysis.subdirpath(), "/", "mig-yes-no.pdf")
}

fpath.for.mig.detection <- function() {
	paste0(analysis.subdirpath(), "/", "mig-detection.txt")
}


